@@ -1,7 +1,7 @@
 package Pcore::WebSocket::Handle;
 
 use Pcore -const, -role, -result;
-use Pcore::Util::Scalar qw[refaddr];
+use Pcore::Util::Scalar qw[weaken refaddr];
 use Pcore::Util::Text qw[decode_utf8 encode_utf8];
 use Pcore::Util::Data qw[to_b64 to_xor];
 use Compress::Raw::Zlib;
@@ -14,16 +14,16 @@ use Pcore::Util::Digest qw[sha1];
 # https://tools.ietf.org/html/rfc7692#page-10
 # https://www.igvita.com/2013/11/27/configuring-and-optimizing-websocket-compression/
 
-requires qw[protocol before_connect_server before_connect_client on_connect_server on_connect_client on_disconnect on_text on_binary on_pong];
+requires qw[protocol before_connect_server before_connect_client on_connect_server on_connect_client on_disconnect on_text on_binary];
 
 has max_message_size => ( is => 'ro', isa => PositiveOrZeroInt, default => 1_024 * 1_024 * 100 );    # 0 - do not check message size
 has pong_interval    => ( is => 'ro', isa => PositiveOrZeroInt, default => 0 );                      # 0 - do not pong automatically
 has compression      => ( is => 'ro', isa => Bool,              default => 0 );                      # use permessage_deflate compression
+has on_disconnect => ( is => 'ro', isa => Maybe [CodeRef], reader => undef );                        # ($ws, $status)
+has on_ping       => ( is => 'ro', isa => Maybe [CodeRef], reader => undef );                        # ($ws, $status)
+has on_pong       => ( is => 'ro', isa => Maybe [CodeRef], reader => undef );                        # ($ws, $status)
 
-has on_disconnect => ( is => 'ro', isa => CodeRef, reader => undef );
-
-has h => ( is => 'ro', isa => InstanceOf ['Pcore::AE::Handle2'], required => 1 );
-
+has h => ( is => 'ro', isa => InstanceOf ['Pcore::AE::Handle2'], init_arg => undef );
 has is_connected => ( is => 'ro', isa => Bool, default => 0, init_arg => undef );
 
 # mask data on send, for websocket client only
@@ -65,6 +65,14 @@ const our $WEBSOCKET_STATUS_REASON => {
     1015 => 'TLS handshake',
 };
 
+sub DEMOLISH ( $self, $global ) {
+    if ( !$global ) {
+        $self->disconnect( result [ 1001, $WEBSOCKET_STATUS_REASON ] );
+    }
+
+    return;
+}
+
 sub send_text ( $self, $data_ref ) {
     $self->{h}->push_write( $self->_build_frame( 1, $self->{compression}, 0, 0, $WEBSOCKET_OP_TEXT, $data_ref ) );
 
@@ -77,13 +85,13 @@ sub send_binary ( $self, $data_ref ) {
     return;
 }
 
-sub ping ( $self, $payload = $WEBSOCKET_PING_PONG_PAYLOAD ) {
+sub send_ping ( $self, $payload = $WEBSOCKET_PING_PONG_PAYLOAD ) {
     $self->{h}->push_write( $self->_build_frame( 1, 0, 0, 0, $WEBSOCKET_OP_PING, \$payload ) );
 
     return;
 }
 
-sub pong ( $self, $payload = $WEBSOCKET_PING_PONG_PAYLOAD ) {
+sub send_pong ( $self, $payload = $WEBSOCKET_PING_PONG_PAYLOAD ) {
     $self->{h}->push_write( $self->_build_frame( 1, 0, 0, 0, $WEBSOCKET_OP_PONG, \$payload ) );
 
     return;
@@ -106,7 +114,7 @@ sub disconnect ( $self, $status = undef ) {
     # destroy handle
     $self->{h}->destroy;
 
-    # remove from cache
+    # remove from cache, affect only server handles
     delete $Pcore::WebSocket::HANDLE->{ refaddr $self};
 
     # call protocol on_disconnect
@@ -123,15 +131,19 @@ sub get_challenge ( $self, $key ) {
     return to_b64( sha1( ($key) . $WEBSOCKET_GUID ), q[] );
 }
 
-sub on_connect ( $self ) {
+sub on_connect ( $self, $h ) {
     return if $self->{is_connected};
 
     $self->{is_connected} = 1;
 
+    $self->{h} = $h;
+
+    weaken $self;
+
     # set on_error handler
     $self->{h}->on_error(
         sub ( $h, @ ) {
-            $self->disconnect( result [ 1001, $WEBSOCKET_STATUS_REASON ] );    # 1001 - Going Away
+            $self->disconnect( result [ 1001, $WEBSOCKET_STATUS_REASON ] ) if $self;    # 1001 - Going Away
 
             return;
         }
@@ -225,7 +237,7 @@ sub on_connect ( $self ) {
     if ( my $pong_interval = $self->pong_interval ) {
         $self->{h}->on_timeout(
             sub ($h) {
-                $self->pong;
+                $self->send_pong;
 
                 return;
             }
@@ -279,7 +291,6 @@ sub _on_frame ( $self, $header, $payload_ref ) {
             undef $self->{_msg};
         }
 
-        # dispatch message
         # TEXT message
         if ( $header->{op} == $WEBSOCKET_OP_TEXT ) {
             if ($payload_ref) {
@@ -313,13 +324,15 @@ sub _on_frame ( $self, $header, $payload_ref ) {
         # PING message
         elsif ( $header->{op} == $WEBSOCKET_OP_PING ) {
 
-            # send pong automatically
-            $self->pong( $payload_ref ? $payload_ref->$* : q[] );
+            # reply pong automatically
+            $self->send_pong( $payload_ref ? $payload_ref->$* : q[] );
+
+            $self->{on_ping}->( $self, $payload_ref || \q[] ) if $self->{on_ping};
         }
 
         # PONG message
         elsif ( $header->{op} == $WEBSOCKET_OP_PONG ) {
-            $self->on_pong( $payload_ref || \q[] );
+            $self->{on_pong}->( $self, $payload_ref || \q[] ) if $self->{on_pong};
         }
     }
 
@@ -456,17 +469,17 @@ sub _parse_frame_header ( $self, $buf_ref ) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 80, 86, 329          | Subroutines::ProhibitManyArgs - Too many arguments                                                             |
+## |    3 | 88, 94, 342          | Subroutines::ProhibitManyArgs - Too many arguments                                                             |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
 ## |    3 |                      | Subroutines::ProhibitExcessComplexity                                                                          |
-## |      | 126                  | * Subroutine "on_connect" with high complexity score (26)                                                      |
-## |      | 240                  | * Subroutine "_on_frame" with high complexity score (27)                                                       |
+## |      | 134                  | * Subroutine "on_connect" with high complexity score (27)                                                      |
+## |      | 252                  | * Subroutine "_on_frame" with high complexity score (30)                                                       |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 390, 392             | NamingConventions::ProhibitAmbiguousNames - Ambiguously named variable "second"                                |
+## |    3 | 403, 405             | NamingConventions::ProhibitAmbiguousNames - Ambiguously named variable "second"                                |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 39, 256              | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
+## |    2 | 39, 268              | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    1 | 302                  | CodeLayout::ProhibitParensWithBuiltins - Builtin function called with parentheses                              |
+## |    1 | 313                  | CodeLayout::ProhibitParensWithBuiltins - Builtin function called with parentheses                              |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
